/*
	tping - TruePing - This program does what the ping program does, but with better timing.

	Copyright (C) 2002 Andrea Denzler - http://andreaplanet.com/tping

	This program is free software; you can redistribute it and/or modify it under the terms 
	of the GNU General Public License as published by the Free Software Foundation version 2.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "myTimer.h"
#include <atomic>

int ProgramExitValue=0;

// --------------------------------------------------------------------------------------
//  IP/ICMP Header
// --------------------------------------------------------------------------------------
#pragma pack(1)

#define ICMP_ECHO 8
#define ICMP_ECHOREPLY 0
#define ICMP_TIMEEXCEEDED 11
#define ICMP_MIN 8 // minimum 8 byte icmp packet (just header)
#define IP_DF           0x4000

// -------------------------------------------------------
// The IP header 
// -------------------------------------------------------
/*
typedef struct {
	unsigned int h_len:4;          // length of the header
	unsigned int version:4;        // Version of IP
	unsigned char tos;             // Type of service
	unsigned short total_len;      // total length of the packet
	unsigned short ident;          // unique identifier
	unsigned short frag_and_flags; // flags
	unsigned char  ttl; 
	unsigned char proto;           // protocol (TCP, UDP etc)
	unsigned short checksum;       // IP checksum

	unsigned int sourceIP;
	unsigned int destIP;

} IpHeader;
*/
// IP Header -- RFC 791
typedef struct 
{
	unsigned char  h_len : 4;			// Version and IHL
	unsigned char  version : 4;			// Version and IHL
	u_char	TOS;			// Type Of Service
	short	TotLen;			// Total Length
	short	ID;				// Identification
	short	FlagOff;		// Flags and Fragment Offset
	u_char	TTL;			// Time To Live
	u_char	Protocol;		// Protocol
	u_short	Checksum;		// Checksum
	struct	in_addr iaSrc;	// Internet Address - Source
	struct	in_addr iaDst;	// Internet Address - Destination
} IpHeader;


// -------------------------------------------------------
// ICMP header minimum fields
// -------------------------------------------------------
typedef struct {
  BYTE i_type;
  BYTE i_code; /* type sub code */
  USHORT i_cksum;
  USHORT i_id;
  USHORT i_seq;
} IcmpHeader;

// -------------------------------------------------------
// ICMP header in case Time Exceeded
// -------------------------------------------------------
typedef struct {
	BYTE i_type;
	BYTE i_code; /* type sub code */
	USHORT i_cksum;
	USHORT not_used1;
	USHORT not_used2;
	// Original IP Header 
	IpHeader	OriginalIP;
	// Original ICMP Data
	IcmpHeader	OriginalICMP;
} IcmpHeaderTimeExceeded;

#define RECEIVE_SIZE	5000

// --------------------------------------------------------------------------------------
// Memory Allocation
// --------------------------------------------------------------------------------------
#define xmalloc(s) HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,(s))
#define xfree(p)   HeapFree (GetProcessHeap(),0,(p))



// --------------------------------------------------------------------------------------
//	ICMP
// --------------------------------------------------------------------------------------
class myICMP
{
private:
	SOCKET				m_SockRaw;
	unsigned			m_Sequence;
	char				*m_ReceiveBuffer;
	struct sockaddr_in	m_HostAddress;
	double				m_ElapsedTime;
	double				m_Jitter;
	//long				m_ElapsedTick;
	long				m_TimeoutRec;
	BOOL				m_PrintTrace;
	BOOL				m_PrintTimeStamp;
	char				m_strHostIP[255];
	unsigned long		m_intHostIP;
	u_char				m_TTL;
	u_char				m_TOS;
	BOOL				m_DontFragment;

public:
	//---------------------------------------------------------------------
	// Init
	//---------------------------------------------------------------------
	inline myICMP()
	{
		m_Sequence=1;
		m_SockRaw=0;
		m_ReceiveBuffer=NULL;
		m_HostAddress.sin_addr.s_addr = 0;
		m_HostAddress.sin_family = AF_INET;
		m_ElapsedTime=0;
		//m_ElapsedTick=0;
		m_TimeoutRec=1000;
		m_PrintTrace=FALSE;
		*m_strHostIP = 0;
		m_intHostIP = 0;
		m_TOS = 0;
		m_TTL = 255;
		m_DontFragment = FALSE;
		m_Jitter=0;
		m_PrintTimeStamp = FALSE;
	}
	inline void Clear() 
	{
		if (m_ReceiveBuffer) {
			xfree(m_ReceiveBuffer);
			m_ReceiveBuffer=NULL;
		}
	}
	inline ~myICMP()
	{
		CloseSocket();
		Clear();
	}

	inline static BOOL InitLibrary()
	{
		WSADATA wsaData;
		if (WSAStartup(MAKEWORD(2,1),&wsaData) != 0) {
			fprintf(stderr,"Windows Socket Init failed: %d\n",GetLastError());
			return FALSE;
		}
		return TRUE;
	}
	inline static void TerminateLibrary()
	{
		WSACleanup();
	}

	inline BOOL OpenSocket()
	{
		m_SockRaw = WSASocket (AF_INET,
					   SOCK_RAW,
					   IPPROTO_ICMP,
					   NULL, 
					   0,
					   WSA_FLAG_OVERLAPPED);
		if (m_SockRaw == INVALID_SOCKET) {
			fprintf(stderr,"Windows Socket Open failed: %d\n",WSAGetLastError());
			return FALSE;
		}
		// I will send a complete IP packet with header
		int on=1;
		int result= setsockopt(m_SockRaw,IPPROTO_IP, IP_HDRINCL,(char *)&on, sizeof(on));
		if (result == SOCKET_ERROR) {
	  		fprintf(stderr,"Windows Socket Init Failed: %d\n",WSAGetLastError());
			return FALSE;
		}
	  return TRUE;
	}
	inline void CloseSocket()
	{
		if (m_SockRaw) {
			closesocket(m_SockRaw);
			m_SockRaw=0;
		}
	}

	inline BOOL SetTimeoutSend(int Timeout_ms)
	{
		int result= setsockopt(m_SockRaw,SOL_SOCKET,SO_SNDTIMEO,(char*)&Timeout_ms,sizeof(Timeout_ms));
		if (result == SOCKET_ERROR) {
	  		fprintf(stderr,"Windows Socket Set SendTimeout to %d ms failed: %d\n",Timeout_ms,WSAGetLastError());
			return FALSE;
		}
		return TRUE;
	}
	inline BOOL SetTimeoutReceive(int Timeout_ms)
	{
		int result= setsockopt(m_SockRaw,SOL_SOCKET,SO_RCVTIMEO,(char*)&Timeout_ms,sizeof(Timeout_ms));
		if (result == SOCKET_ERROR) {
	  		fprintf(stderr,"Windows Socket Set ReceiveTimeout to %d ms failed: %d\n",Timeout_ms,WSAGetLastError());
			return FALSE;
		}
		m_TimeoutRec=Timeout_ms;
		return TRUE;
	}
	inline BOOL SetTimeToLive(u_char TTL)
	{
		m_TTL = TTL;
		int result= setsockopt(m_SockRaw,IPPROTO_IP,IP_TTL,(char*)&TTL,sizeof(TTL));
		if (result == SOCKET_ERROR) {
	  		fprintf(stderr,"Windows Socket Set TimeToLive %d failed: %d\n",TTL,WSAGetLastError());
			return FALSE;
		}
		return TRUE;
	}
	inline BOOL SetTypeOfService(u_char TOS)
	{
		m_TOS = TOS;
		int result= setsockopt(m_SockRaw,IPPROTO_IP,IP_TOS,(char*)&TOS,sizeof(TOS));
		if (result == SOCKET_ERROR) {
	  		fprintf(stderr,"Windows Socket Set TypeOfService %d failed: %d\n",TOS,WSAGetLastError());
			return FALSE;
		}
		fprintf(stderr,"Windows Socket Set TypeOfService %d OK\n",TOS);
		return TRUE;
	}
	inline void SetDontFragmentFlag(BOOL dontFragment) 
	{
		m_DontFragment = dontFragment;
	}
	inline BOOL SetPrintTrace(BOOL PrintTraceStyle)
	{
		m_PrintTrace=PrintTraceStyle;
		return TRUE;
	}
	inline BOOL GetSequence() 
	{
		return m_Sequence;
	}
	inline BOOL SetSequence(USHORT NewSequence) 
	{
		m_Sequence=NewSequence;
	}
	inline double GetElapsedTime()
	{
		return m_ElapsedTime;
	}
	/*inline double GetElapsedTick()
	{
		return m_ElapsedTick;
	}*/

	//---------------------------------------------------------------------
	// Set Current Host
	// output su ResolvedHost
	//---------------------------------------------------------------------
	inline BOOL SetHost(const char *Host, char *HostIP=NULL, char *HostName=NULL, unsigned long SourceIPAddress=0)
	{
		BOOL retVal=GetAddress(m_HostAddress,Host,HostIP,HostName);

		struct sockaddr_in souraddr;
		m_intHostIP = 0; // error!
		if (SourceIPAddress != 0) {
			m_intHostIP = SourceIPAddress;
		} else if (retVal && get_localaddress_from_route(&m_HostAddress,&souraddr))
			m_intHostIP = souraddr.sin_addr.S_un.S_addr;
		else {
			// acquisisci hostip di default
			char strHost[512];
			*strHost=0;
			gethostname(strHost,sizeof(strHost));
			struct hostent FAR *HostInfo=gethostbyname(strHost);
			if (HostInfo != NULL && HostInfo->h_addr_list[0] != 0) {
				m_intHostIP = *(unsigned long *)HostInfo->h_addr_list[0];
			}
		}
		strcpy(m_strHostIP,inet_ntoa(*(struct in_addr *)&m_intHostIP));
		return retVal;
	}
	inline const char *GetSourceIP() 
	{
		return m_strHostIP;
	}
	inline BOOL GetAddress(struct sockaddr_in &destaddr, const char *Host, char *HostIP=NULL, char *HostName=NULL)
	{
		struct hostent		*hp;
		unsigned int		addr=INADDR_NONE;

		destaddr.sin_addr.s_addr = 0;
		destaddr.sin_family = AF_INET;
		// -----------------------------------------------------------------
		// Get HOST address
		// -----------------------------------------------------------------
		hp = gethostbyname(Host);
		if (hp == NULL) {
			addr = inet_addr(Host);
		}
		if ((hp == NULL) && (addr == INADDR_NONE) ) {
			fprintf(stderr,"Windows Socket Unable to Resolve Host %s\n",Host);
			if (HostIP) *HostIP=0;
			if (HostName) *HostName=0;
			return FALSE;
		}
		if (hp != NULL) {
			memcpy(&(destaddr.sin_addr),hp->h_addr,hp->h_length);
			destaddr.sin_family = hp->h_addrtype;
			if (HostName) strcpy(HostName,hp->h_name);
		}
		else {
  			destaddr.sin_addr.s_addr = addr;
			destaddr.sin_family = AF_INET;
			if (HostName) strcpy(HostName,inet_ntoa(destaddr.sin_addr));
		}
		if (HostIP) strcpy(HostIP,inet_ntoa(destaddr.sin_addr));
		return TRUE;
	}


	inline void SetPrintTimeStampOption(BOOL boolPrintTimeStamp)
	{
		m_PrintTimeStamp=boolPrintTimeStamp;
	}
	inline void PrintTimeStamp()
	{
		if (!m_PrintTimeStamp) 
			return;
		// get current time (approximate calculation)
		struct _timeb timebuffer;
		_ftime( &timebuffer );
		struct tm *t = localtime( & timebuffer.time );
		printf( "%04ld-%02ld-%02ld %02d:%02d:%02d.%03d ",(long)t->tm_year+1900,(long)t->tm_mon+1,(long)t->tm_mday,t->tm_hour,t->tm_min,t->tm_sec,timebuffer.millitm);
	}

	inline BOOL IPStringToIPNumeric(const char *HostIP,char *ip4)
	{
		const char	*p;
		long	n;
		// get first number
		p=HostIP; n=0;
		for (; *p!=0; ++p) {
			if (*p >= '0' && *p <= '9')
				n = n*10 + *p - '0';
			else 
				break;
		}
		if (n < 0 || n >= 255)
			return FALSE;
		ip4[0]=(char )n;
		if (*p++ != '.') 
			return FALSE;
		// get second number
		n=0;
		for (; *p!=0; ++p) {
			if (*p >= '0' && *p <= '9')
				n = n*10 + *p - '0';
			else 
				break;
		}
		if (n < 0 || n >= 255)
			return FALSE;
		ip4[1]=(char )n;
		if (*p++ != '.') 
			return FALSE;
		// get third number
		n=0;
		for (; *p!=0; ++p) {
			if (*p >= '0' && *p <= '9')
				n = n*10 + *p - '0';
			else 
				break;
		}
		if (n < 0 || n >= 255)
			return FALSE;
		ip4[2]=(char )n;
		if (*p++ != '.') 
			return FALSE;
		// get last number
		n=0;
		for (; *p!=0; ++p) {
			if (*p >= '0' && *p <= '9')
				n = n*10 + *p - '0';
			else 
				break;
		}
		if (n < 0 || n >= 255)
			return FALSE;
		ip4[3]=(char )n;
		if (*p++ != 0) 
			return FALSE;
		// ok
		return TRUE;
	}

	//---------------------------------------------------------------------
	// Get HostName from IP address
	//---------------------------------------------------------------------
	inline BOOL GetHostName(const char *HostIP, char *HostName)
	{
		HOSTENT *namep;
		char	ip4[4];
		strcpy(HostName,HostIP);
		// translate string to ip values
		if (IPStringToIPNumeric(HostIP,ip4) == FALSE)
			return FALSE;
		// get
		namep=gethostbyaddr(ip4,4,AF_INET);
		if (namep == NULL) {
			return FALSE;
		}
		strcpy(HostName,namep->h_name);
		return TRUE;
	}

	//---------------------------------------------------------------------
	// Send a ping to the current host
	// Buffer,BufferSize is any data
	//---------------------------------------------------------------------
	inline BOOL SendEcho(const char *Buffer, u_short BufferSize, myTIMER *mTimer)
	{
		// -----------------------------------------------------------------
		// Memory allocation
		// -----------------------------------------------------------------
		u_short icmpsize = 0;
		u_short ipsize = 0;	// Default Packed Size 0
		IpHeader	*ip_hdr;
		IcmpHeader	*icmp_hdr;
		char *dataPtr;
		icmpsize += sizeof(IcmpHeader);
		icmpsize += BufferSize;
		ipsize = icmpsize+sizeof(IpHeader);
		ip_hdr = (IpHeader *)xmalloc(1024+ipsize);
		if (!ip_hdr) {
			fprintf(stderr,"HeapAlloc failed %d\n",GetLastError());
			return FALSE;
		}
		dataPtr = (char *)ip_hdr + sizeof(IpHeader);
		icmp_hdr = (IcmpHeader *)dataPtr;
		dataPtr += sizeof(IcmpHeader);
		// -----------------------------------------------------------------
		// Fill packet with personal data
		// -----------------------------------------------------------------
		// aggiungi dati utente
		memcpy(dataPtr,Buffer,BufferSize);
		dataPtr += BufferSize;
		// -----------------------------------------------------------------
		// Fill packet with header
		// -----------------------------------------------------------------
		icmp_hdr->i_type = ICMP_ECHO;
		icmp_hdr->i_code = 0;
		icmp_hdr->i_id = (USHORT)GetCurrentProcessId();
		icmp_hdr->i_seq = (USHORT)m_Sequence++;
		icmp_hdr->i_cksum = 0;
		icmp_hdr->i_cksum = GetCheckSum((USHORT*)icmp_hdr,icmpsize);
		// -----------------------------------------------------------------
		// Set IP Header
		// -----------------------------------------------------------------
		ip_hdr->version = 4; 
		ip_hdr->h_len = sizeof(IpHeader)>>2; 
		ip_hdr->TOS = m_TOS; 
		ip_hdr->TotLen = htons(ipsize); 
		ip_hdr->ID = htons(4321);
		ip_hdr->FlagOff = htons(m_DontFragment? IP_DF:0); 
		ip_hdr->TTL = m_TTL; 
		ip_hdr->Protocol = IPPROTO_ICMP; 
		ip_hdr->Checksum = 0;

		ip_hdr->iaSrc.S_un.S_addr = m_intHostIP;
		ip_hdr->iaDst = m_HostAddress.sin_addr;
		ip_hdr->Checksum = GetCheckSum((USHORT*)ip_hdr,ipsize);
		// -----------------------------------------------------------------
		// Start Timer
		// -----------------------------------------------------------------
		mTimer->StartTimer();
		// -----------------------------------------------------------------
		// Send
		// -----------------------------------------------------------------
		int result = sendto(m_SockRaw,(LPCSTR)ip_hdr,ipsize,0,(struct sockaddr *)&m_HostAddress,sizeof(m_HostAddress));
		if (result == SOCKET_ERROR) {
			PrintTimeStamp(); 
			int err = WSAGetLastError();
			switch (err) {
				case WSAETIMEDOUT:
	  				printf("Sending Timed out.\n"); break;
				case WSAEHOSTUNREACH:
	  				printf("Destination host unreachable.\n"); break;
				case WSAENETUNREACH:
	  				printf("Destination network unreachable.\n"); break;
				case WSAEINTR:
					WSASetLastError(0);
	  				printf("Socket error 'sendto was cancelled'.\n"); break;
				default:
					fprintf(stderr,"Send failed: %d\n",err);
				}
			xfree(ip_hdr);
			return FALSE;
		}
		if (result != ipsize) {
			PrintTimeStamp(); 
			fprintf(stdout,"Send failed partially, wrote %d bytes\n",result);
			xfree(ip_hdr);
			return FALSE;
		}
		xfree(ip_hdr);
		return TRUE;
	}


	//---------------------------------------------------------------------
	// Initialize listener
	//---------------------------------------------------------------------
	inline BOOL ListenSetup(int Timeout_ms) 
	{
		// Init Memory
		Clear();
		m_ReceiveBuffer = (char *)xmalloc(RECEIVE_SIZE);
		if (m_ReceiveBuffer == NULL) {
			fprintf(stderr,"HeapAlloc failed %d\n",GetLastError());
			return FALSE;
		}
		// Bind
		struct sockaddr_in local;
		local.sin_family = AF_INET;
		local.sin_addr.s_addr = INADDR_ANY; 
		local.sin_port = 0;
		int result= bind(m_SockRaw,(struct sockaddr*)&local,sizeof(local));
	    if(result == SOCKET_ERROR) {
  			fprintf(stderr,"failed to bind: %d\n",WSAGetLastError());
			return FALSE;
		}
		// set timeout
		return SetTimeoutReceive(Timeout_ms);
	}


	//---------------------------------------------------------------------
	// Receive Ping
	// -1 --> error
	//  0 --> in caso di timeout
	//  1 --> packet received
	// WaitForSequence=NULL any sequence
	//---------------------------------------------------------------------
	inline int Listen(myTIMER *mTimer, std::atomic_int *WaitForSequence=NULL, char *Buffer=NULL, int *BufferSize=NULL, BOOL FlagPrint=TRUE)
	{
		struct sockaddr_in from;
		int err;
		int fromlen = sizeof(from);
		const char *sResponse;
		int	IsOurHeader;
		//-----------------------------------------------------------------------------------------------
		// init a 0 at default
		//-----------------------------------------------------------------------------------------------
		int MaxSize=0;
		if (BufferSize) {
			MaxSize = *BufferSize;
			*BufferSize = 0;
		}
		//-----------------------------------------------------------------------------------------------
		// Receive Packet
		//-----------------------------------------------------------------------------------------------		
		wait:;
		int result = recvfrom(m_SockRaw,m_ReceiveBuffer,RECEIVE_SIZE,0,(struct sockaddr*)&from,&fromlen);
		if (result == SOCKET_ERROR) {
			err=WSAGetLastError();
			// no anser?
			if (err == WSAETIMEDOUT) {	
				timeout_label:;
 			    if (FlagPrint) { PrintTimeStamp(); printf("Request timed out.\n"); }
				return 0;
			}
			if (err == WSAEMSGSIZE) {
				// if packet too big, just read the first part
				result = RECEIVE_SIZE-1;	// max size
			} else {
				if (FlagPrint) { PrintTimeStamp(); fprintf(stderr,"Receiving data failed: %d\n",WSAGetLastError()); }
				return -1;
			}
		}
		sResponse = "Reply";
		IsOurHeader=FALSE;
		//-----------------------------------------------------------------------------------------------
		// read TIMER as soon as possible
		//-----------------------------------------------------------------------------------------------
		double oldElapsedTime=m_ElapsedTime;
		m_ElapsedTime = mTimer->Stop();	// msec
		//m_ElapsedTick = mTimer->TimeTick();	// tick msec
		//-----------------------------------------------------------------------------------------------
		// Analyze Packet, if IP read header
		//-----------------------------------------------------------------------------------------------
		IpHeader		*iphdr;
		IcmpHeader		*icmphdr;
		unsigned short	iphdrlen;
		iphdr = (IpHeader *)m_ReceiveBuffer;
		iphdrlen = (USHORT)(iphdr->h_len * 4) ; // number of 32-bit words *4 = bytes
		if (result  < iphdrlen + ICMP_MIN) {
			if (FlagPrint) { PrintTimeStamp(); printf("Received partial packet from %s\n",inet_ntoa(from.sin_addr)); }
			return 0;
		}
		icmphdr = (IcmpHeader*)(m_ReceiveBuffer + iphdrlen);
		//-----------------------------------------------------------------------------------------------
		// Header is valid?
		//-----------------------------------------------------------------------------------------------
		// if it is not a ping ignore it
		switch (icmphdr->i_type) {
			// Ok, answer
			case ICMP_ECHOREPLY:
				ProgramExitValue=1;
				break;
			// expired
			case ICMP_TIMEEXCEEDED: 
			{
				// analyze ip
				IcmpHeaderTimeExceeded *ipte = (IcmpHeaderTimeExceeded *)icmphdr;
				if (ipte->OriginalICMP.i_id == (USHORT)GetCurrentProcessId() &&
					(WaitForSequence == NULL || ipte->OriginalICMP.i_seq == *WaitForSequence)) {
					if (ipte->i_code == 1)
						{ if (FlagPrint)  { PrintTimeStamp(); printf("Fragment reassembly time exceeded.\n"); return 0; } }
					else
						{ IsOurHeader=TRUE; sResponse = m_PrintTrace?"Trace":"Reply TTL expired"; }
					break;
				}
				// it's not our packet
			}
			default:
				if (m_ElapsedTime >= m_TimeoutRec)
					goto timeout_label;
				goto wait;
		}
		// if we don't sent it ignore it also
		if (IsOurHeader == FALSE && icmphdr->i_id != (USHORT)GetCurrentProcessId()) {
			// wait again until timeout
			if (m_ElapsedTime >= m_TimeoutRec)
				goto timeout_label;
			goto wait;
		}
		//-----------------------------------------------------------------------------------------------
		// It's the packet we was waiting?
		//-----------------------------------------------------------------------------------------------
		if (IsOurHeader == FALSE && WaitForSequence != NULL && icmphdr->i_seq != *WaitForSequence) {
			// wrong sequence, probablyh old packet
			// so lets wait again
			if (m_ElapsedTime >= m_TimeoutRec)
				goto timeout_label;
			goto wait;
		}
		//-----------------------------------------------------------------------------------------------
		// Get Data Buffer
		//-----------------------------------------------------------------------------------------------
		char *dataPtr = (char *)iphdr + iphdrlen + sizeof(IcmpHeader);
		int dataSize = result - iphdrlen - sizeof(IcmpHeader);
		//-----------------------------------------------------------------------------------------------
		// Update realtime Jitter as in RFC 1889
		//-----------------------------------------------------------------------------------------------
		// J = J+(|D(i-1,i)|-J)/16
		if (oldElapsedTime > 0 && m_ElapsedTime > 0) {
			m_Jitter += (fabs(m_ElapsedTime-oldElapsedTime)-m_Jitter)/16.0;
		}
		//-----------------------------------------------------------------------------------------------
		// Print Output
		//-----------------------------------------------------------------------------------------------
		if (FlagPrint) {
			PrintTimeStamp(); 
			printf("%s from %s: bytes=%d time %.2lf ms, TTL=%d, Hop=%d, Jitter=%.2lf ms\n",
			sResponse,
			inet_ntoa(from.sin_addr),
			result - iphdrlen - ICMP_MIN, // print data size
			//iphdr->total_len * 4 - iphdrlen - ICMP_MIN, // mettiamo i byte originali nel pacchetto
			m_ElapsedTime,
			//m_ElapsedTick,
			iphdr->TTL,
			(iphdr->TTL>128? 255 : (iphdr->TTL>64? 128 : (iphdr->TTL>31? 64:31) )) - iphdr->TTL,
			m_Jitter
			);
		}
		//-----------------------------------------------------------------------------------------------
		// Return the buffer
		//-----------------------------------------------------------------------------------------------
		if (BufferSize) {
			if (dataSize < MaxSize) 
				MaxSize=dataSize;
			memcpy(Buffer,dataPtr,MaxSize);
		}
		return 1;
	}

private:
	// -----------------------------------------------------------------
	// calc CheckSum
	// -----------------------------------------------------------------
	static inline USHORT GetCheckSum(USHORT *buffer, int size) 
	{
		unsigned long cksum=0;
		while(size >1) {
			cksum+=*buffer++;
			size -=sizeof(USHORT);
		}

		if(size ) {
			cksum += *(UCHAR*)buffer;
		}
		cksum = (cksum >> 16) + (cksum & 0xffff);
		cksum += (cksum >>16);
		return (USHORT)(~cksum);
	}

	// Get the correct local address for the destination 'dest'
	static inline BOOL get_localaddress_from_route(struct sockaddr_in *dest, struct sockaddr_in *localaddress)
	{
		int sock_rt, len;
		char on=1;
		struct sockaddr_in localaddress_test;

		memset(&localaddress_test, 0, sizeof(localaddress_test));
		sock_rt = socket(AF_INET, SOCK_DGRAM, 0 );

		dest->sin_port = htons(11111);
		if (setsockopt(sock_rt, SOL_SOCKET, SO_BROADCAST, &on, sizeof(on)) == -1) {
			closesocket(sock_rt);
			return FALSE; // error
		}
		if (connect(sock_rt, (struct sockaddr*)dest, sizeof(struct sockaddr_in)) == -1 ) {
			closesocket(sock_rt);
			return FALSE; // error
		}
		len = sizeof(localaddress_test);
		if (getsockname(sock_rt, (struct sockaddr *)&localaddress_test, &len) == -1 ) {
			closesocket(sock_rt);
			return FALSE; // error
		}
		closesocket(sock_rt);
		if (localaddress_test.sin_addr.s_addr == 0)
			return FALSE; // ok I did not found the address
		memcpy(localaddress, &localaddress_test, sizeof(struct sockaddr_in));
		return TRUE;
	}
};



