/*
	tping - TruePing - This program does what the ping program does, but with better timing.

	Copyright (C) 2002 Andrea Denzler - http://andreaplanet.com/tping

	This program is free software; you can redistribute it and/or modify it under the terms 
	of the GNU General Public License as published by the Free Software Foundation version 2.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <atomic>

class myTIMER
{
private:
	LARGE_INTEGER				Frequency;				// CPU frequency
	std::atomic<LONGLONG>		CounterStart;			// high resolution counter (thread safe)
	//LARGE_INTEGER	CounterStop;			// high resolution counter
	//long			TickStart;				// low resolution counter
	//long			TickElapsed;
	//LARGE_INTEGER	CounterStartAdd;		// Start/Stop execution time (very low value)

public:
	myTIMER()
	{
		// calculate OS CPU performance 
		if (!QueryPerformanceFrequency(&Frequency)) {
			fprintf(stderr,"Unable to get System Frequency %d\n",GetLastError());
			Frequency.QuadPart=0;
		}
		//CounterStartAdd.QuadPart=0;
		// calculate the time necessary to do a Start/Stop. very low value,
		// but I like to decrease this value when calculating the effetive value.
		//StartTimer();
		//Stop();
		//CounterStartAdd.QuadPart = CounterStop.QuadPart - CounterStart.QuadPart;
	}


	void StartTimer()
	{
		// read timer using standard
		//TickStart = GetTickCount();
		// read counter
		LARGE_INTEGER s;
		if (!QueryPerformanceCounter(&s)) {
			fprintf(stderr,"Unable to get System Time %d\n",GetLastError());
			CounterStart = 0;
			return;
		}
		CounterStart = s.QuadPart;
		//CounterStart.QuadPart += CounterStartAdd.QuadPart;
	}

	double Stop()
	{
		LARGE_INTEGER	CounterStop;			// high resolution counter
		if (CounterStart == 0 || Frequency.QuadPart == 0)
			return -1;
		// read counter
		if (!QueryPerformanceCounter(&CounterStop)) {
			fprintf(stderr,"Unable to get System Time %d\n",GetLastError());
			return -1;
		}
		// read timer using standard
		//TickElapsed = GetTickCount() - TickStart;
		// calculate high resolution counter
		return (CounterStop.QuadPart - CounterStart)*(double)1000.0/(double)Frequency.QuadPart;
	}

	/*long TimeTick()
	{
		return TickElapsed;
	}*/

};
