/*
	tping - TruePing - This program does what the ping program does, but with better timing.

	Copyright (C) 2002 Andrea Denzler - http://andreaplanet.com/tping

	This program is free software; you can redistribute it and/or modify it under the terms 
	of the GNU General Public License as published by the Free Software Foundation version 2.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program; 
	if not, write to the Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#include "stdafx.h"
#include "myICMP.h"
#include <atomic>

// I'know too many static variables....very ugly

// this to synchronize the receiving thread with sending thread (we play ping-pong)
// I know, it's better to use semaphores, but in this case it's ok...
typedef enum
{
	STATUS_LISTENER,	// next step to the listenerTocca al listener la mossa successiva
	STATUS_SENDER,		// next step to the sender
	STATUS_EXIT
} StatusType;
static std::atomic<StatusType> Status;
static std::atomic_bool RequestStop=FALSE;
static std::atomic_bool PrintTraceStyle=FALSE;
static std::atomic_bool FlagPrintTimeStamp=FALSE;

// the packet number we are actually waiting. Else I may receive something very old.
static std::atomic_int WaitSequence=-1;

// timeout default ms
static int Timeout=1000;

// how long it took?
static myTIMER GoodTimer;

// statistics
static std::atomic_long SendCount = 0;
static std::atomic_long ReceiveCount = 0;
static std::atomic_long JitterCount = 0;
static std::atomic<double> ElapsedMin=-1;
static std::atomic<double> ElapsedMax=-1;
static std::atomic<double> ElapsedAverage=-1;
static std::atomic_int SkipFirstPackets=0;
#define	JITTER_ARRAY_SIZE	100
// added by JJB: array to collect Jitter statistics
static std::atomic<double> JitterArray[JITTER_ARRAY_SIZE];
static char HostIP[50];

DWORD WINAPI ReceiveFunc(VOID);
void PrintUsage();
void FillDataBuffer(char *buffer, int size, BOOL FlagPingCompatible);
DWORD WINAPI DoNothing();
void PrintStatistics();

#define MAX_DATASIZE	65535


BOOL WINAPI CtrlHandler(DWORD fdwCtrlType) 
{ 
    switch (fdwCtrlType) 
    { 
        // Handle the CTRL+C signal.
        case CTRL_C_EVENT: 
			// Exit
			//PrintStatistics();
			Status = STATUS_EXIT;
            return TRUE; 
 
        case CTRL_BREAK_EVENT: 
			PrintStatistics();
			printf("\n");
			// Print Report and continue
			return TRUE;

        // Pass other signals to the next handler. 

/* 
        case CTRL_LOGOFF_EVENT: 
			printf("Logoff\r\n");
			return TRUE;
 
        case CTRL_SHUTDOWN_EVENT: 
			printf("Shutdown\r\n");
			return TRUE;

        // CTRL+CLOSE: confirm that the user wants to exit. 
        case CTRL_CLOSE_EVENT: 
			printf("Close\r\n");
            return TRUE; 
*/	
        default: 
            return FALSE; 
    } 
} 

template <typename T>
bool argvGetNextValue(int argc, char* argv[], int *index, T *value)
{
	if (argv[*index][2] != 0)
		*value = atoi(&argv[*index][2]);
	else
		if (*index + 1 < argc)
			*value = atoi(argv[++*index]);
		else
			return false;
	return true;
}

int __cdecl main(int argc, char* argv[])
{
	int	i;
	const char *host=NULL,*hostinput;
	char hostname[150];
	const char *sourceAddressPtr=NULL;
	char buffer[MAX_DATASIZE];
	myICMP	myping;
	myICMP	mylisten;
	HANDLE  hThread = NULL, hThread2 = NULL;
	DWORD	ThreadResult;
	BOOL	FlagUseThread=TRUE;
	BOOL	FlagPingCompatible=FALSE;
	BOOL	HostReverseResolve=FALSE;
	BOOL	DontFragmentFlag=FALSE;


	long	PacketCount=0; // default 0
	int  	TimeToLive=-1;
	u_char	TimeToLiveMax=128;
	u_short	DataSize=32;
	int		PingDelay=700;
	int		TypeOfService=-1;
	unsigned long SourceIPAddress = 0;

	// Analize input
	hostinput=NULL;
	for (i=1; i<argc; i++) {
		// if no option then it's the host
		if (*argv[i] != '-') {
			if (!hostinput) 
				hostinput=argv[i];
			else {
				printf("Invalid parameter: %s\n",argv[i]);
				return 0;
			}
			continue;
		}
		// it' an option
		// -t -> never end ping
		if (argv[i][1] == 't') {
			PacketCount = -1;
			continue;
		}
		// -a resolves numeric ip to address
		if (argv[i][1] == 'a') {
			HostReverseResolve = TRUE;
			continue;
		}
		// timeout
		if (argv[i][1] == 'w') {
			if (!argvGetNextValue(argc, argv, &i, &Timeout)) {
				printf("Value must be supplied for option -w.\n");
				return 0;
			}
			if (Timeout < 1 || Timeout > 65535) {
				printf("Bad value for option -w, valid range is from 1 to 65535.\n");
				return 0;
			}
			continue;
		}
		// packetcount
		if (argv[i][1] == 'n') {
			if (!argvGetNextValue(argc, argv, &i, &PacketCount)) {
				printf("Value must be supplied for option -n.\n");
				return 0;
			}
			continue;
		}
		// TTL
		if (argv[i][1] == 'i') {
			int TimeToLiveMax1;
			if (!argvGetNextValue(argc, argv, &i, &TimeToLiveMax1)) {
				printf("Value must be supplied for option -i.\n");
				return 0;
			}
			if (TimeToLiveMax1 < 0 || TimeToLiveMax1 > 255) {
				printf("Bad value for option -i, valid range is from 0 to 255.\n");
				return 0;
			}
			TimeToLiveMax = static_cast<u_char>(TimeToLiveMax1);
			continue;
		}
		// flag ping compatible or random data (default)
		if (argv[i][1] == 'j') {
			TimeToLive=1;
			PrintTraceStyle=TRUE;
			continue;
		}
		// type of service
		if (argv[i][1] == 'v') {
			if (!argvGetNextValue(argc, argv, &i, &TypeOfService)) {
				printf("Value must be supplied for option -v.\n");
				return 0;
			}
			continue;
		}

		// datasize
		if (argv[i][1] == 'l') {
			int DataSizei;
			if (!argvGetNextValue(argc, argv, &i, &DataSizei)) {
				printf("Value must be supplied for option -l.\n");
				return 0;
			}
			if (DataSizei < 0 || DataSizei > MAX_DATASIZE) {
				printf("Bad value for option -l, valid range is from 0 to %d.\n",MAX_DATASIZE);
				return 0;
			}
			DataSize = static_cast<u_short>(DataSizei);
			continue;
		}
		// flag thread
		if (argv[i][1] == 'p') {
			FlagUseThread = FALSE;
			continue;
		}
		// don't fragment flag
		if (argv[i][1] == 'f') {
			DontFragmentFlag = TRUE;
			continue;
		}		
		// flag ping compatible or random data (default)
		if (argv[i][1] == 'c') {
			FlagPingCompatible = TRUE;
			continue;
		}
		// timestamp option
		if (argv[i][1] == 'h') {
			FlagPrintTimeStamp = TRUE;
			continue;
		}
		// delay between pings
		if (argv[i][1] == 'd') {
			if (!argvGetNextValue(argc, argv, &i, &PingDelay)) {
				printf("Value must be supplied for option -d.\n");
				return 0;
			}
			if (PingDelay <= 0) PingDelay = 0;
			if (PingDelay > 60000) PingDelay = 60000;
			continue;
		}
		// skip n packets in statistic calculation
		if (argv[i][1] == 'z') {
			if (!argvGetNextValue(argc, argv, &i, &SkipFirstPackets)) {
				SkipFirstPackets = 1;
			}
			if (SkipFirstPackets < 0 || SkipFirstPackets > 65535) {
				printf("Bad value for option -z, valid range is from 0 to 65535.\n");
				return 0;
			}
			continue;
		}
		// -S srcaddr     Source address to use.
		if (argv[i][1] == 'S') {
			SourceIPAddress = 0;
			if (++i > argc) {
				printf("Value must be supplied for option -S.\n");
				return 0;
			}
			sourceAddressPtr = argv[i];
			continue;
		}
		// error
		printf("Invalid option %s\n",argv[i]);
		return 0;
	}
	if (!hostinput) {
		PrintUsage();
		return 0;
	}
	if (PacketCount == 0) {
		PacketCount = TimeToLive <= 0? 5 : 35;
	}
	if (TimeToLive<=0) 
		TimeToLive=TimeToLiveMax;

	// inizialize random generator
    srand( (unsigned)GetTickCount() );

	// Init
	if (!myICMP::InitLibrary()) 
		return 0;

	// set control handler
    if (!SetConsoleCtrlHandler( 
        (PHANDLER_ROUTINE) CtrlHandler,  // handler function 
        TRUE)) {                           // add to list 
        printf("Could not set control handler. Ctrl+C/Break exception is disabled.\n"); 
	}

	// Open Socket
	if (!myping.OpenSocket()) {
		myICMP::TerminateLibrary();
		return 0;
	}
	if (!myping.SetTimeoutSend(Timeout))
		goto exit;
	if (!myping.SetTimeToLive(TimeToLiveMax))
		goto exit;
	myping.SetPrintTimeStampOption(FlagPrintTimeStamp);
	myping.SetDontFragmentFlag(DontFragmentFlag);

	// reverse resolve
	if (HostReverseResolve) {
		if (myping.GetHostName(hostinput,hostname))
			hostinput=hostname;
	}

	// resolve source host
	if (sourceAddressPtr != NULL) {
		struct sockaddr_in sourceAddrs;
		if (!myping.GetAddress(sourceAddrs, sourceAddressPtr)) {
			printf("%s is not a valid address.\n", argv[i]);
			return 0;
		}
		SourceIPAddress = sourceAddrs.sin_addr.S_un.S_addr;
	}

	// calculate host?
	if (!myping.SetHost(hostinput,HostIP,hostname, SourceIPAddress))
		goto exit;

	// init
	host=hostname;
	if (strcmp(host,HostIP) == 0)
		printf("\nPinging %s with %d bytes of data from [%s]:\n\n",host,DataSize,myping.GetSourceIP());
	else
		printf("\nPinging %s [%s] with %d bytes of data from [%s]:\n\n",host,HostIP,DataSize,myping.GetSourceIP());

	// Thread2, do nothing
	hThread2 = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)DoNothing,NULL,0,&ThreadResult);
	if (hThread2 == NULL) {
		printf("Thread creation error\n");
		goto exit;
	}
	BOOL boolSendOK;

	// Ping Ping Ping
	if (FlagUseThread) {
		// next step to the listener. he will change status.
		Status = STATUS_LISTENER;
		// create a receive thread, the listener will start immediately but
		// we want that he is ready before sending any ping.
		hThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ReceiveFunc,NULL,0,&ThreadResult);
		if (hThread == NULL) {
			printf("Thread creation error\n");
			goto exit;
		}
		// set high priority, we dont want to wait too much.
		SetThreadPriority(hThread,THREAD_PRIORITY_HIGHEST);
		BOOL PacketsSend=FALSE;
		// ping!
		for (i=PacketCount; i!=0 && Status != STATUS_EXIT; ) {
			// wait that the sender do his job
			while (Status != STATUS_SENDER) {
				if (Status == STATUS_EXIT)
					break;
				Sleep(1);
			}
			// let's wait before sending another packet
			if (PacketsSend && PingDelay > 0)
				Sleep(PingDelay);
			if (i == 1) 
				Status=STATUS_EXIT;
			// Set TTL
			if (!myping.SetTimeToLive(static_cast<u_char>(TimeToLive)))
				goto exit;
			// Set TOS
			if (TypeOfService >=0 && !myping.SetTypeOfService(static_cast<u_char>(TypeOfService)))
				goto exit;
			// Set Don't fragment Flag
			// Set time to live
			if (TimeToLive<TimeToLiveMax) 
				++TimeToLive;
			// Set packet number WaitSequence
			WaitSequence=myping.GetSequence();
			// fill data buffer with different values if FlagRandomData e' TRUE
			FillDataBuffer(buffer,DataSize,FlagPingCompatible);
			// Send Ping
			boolSendOK=myping.SendEcho(buffer,DataSize,&GoodTimer);
			++SendCount;
			PacketsSend=TRUE;
			// decrease count
			if (i > 0) --i;
			// Now tell listener to wait
			if (boolSendOK && Status != STATUS_EXIT)
				Status = STATUS_LISTENER;
		}

	} else {
		// let's use same socket/thread
		if (!myping.ListenSetup(Timeout))
			goto exit;
		myping.SetPrintTrace(PrintTraceStyle);
		Status = STATUS_LISTENER;

		// ping!
		for (i=PacketCount; i!=0 && Status != STATUS_EXIT; ) {
			// initializze WaitSequence
			WaitSequence=myping.GetSequence();
			// Fill data buffer everytime differently if FlagRandomData e' TRUE
			FillDataBuffer(buffer,DataSize,FlagPingCompatible);
			// Set TTL
			if (!myping.SetTimeToLive(static_cast<u_char>(TimeToLive)))
				goto exit;
			// Set TOS
			if (TypeOfService >=0 && !myping.SetTypeOfService(static_cast<u_char>(TypeOfService)))
				goto exit;
			if (TimeToLive<TimeToLiveMax) ++TimeToLive;
			// Send Ping
			boolSendOK=myping.SendEcho(buffer,DataSize,&GoodTimer);
			++SendCount;
			if (boolSendOK) {
				// Wait...
				int result = myping.Listen(&GoodTimer,&WaitSequence);			
				if (result < 0)
					break;
				// update stats
				if (result > 0) {
					if (SkipFirstPackets > 0) {
						--SkipFirstPackets;
						--SendCount;
					} else {
						double t = myping.GetElapsedTime();
						if (ElapsedMin < 0) {
							// changed by JJB : collect first ping to Jitter
							ElapsedMin = ElapsedMax = ElapsedAverage = JitterArray[JitterCount++] = t;
						} else {
							if (ElapsedMin > t) ElapsedMin = t;
							if (ElapsedMax < t) ElapsedMax = t;
							ElapsedAverage = ElapsedAverage + t;
							// added by JJB : collect upto 100 pings for Jitter
							if (JitterCount < JITTER_ARRAY_SIZE) {
								JitterArray[JitterCount++] = t;
							}
						}
						++ReceiveCount;
					}
				}
			}
			// let's sleep a while
			if (PingDelay > 0) 
				Sleep(PingDelay);
			// decrease count
			if (i > 0) 
				--i;
		}
	}

	Status = STATUS_EXIT;
	// let's wait that thread closes...
	if (hThread) 
		WaitForSingleObject(hThread, INFINITE); 
	if (hThread2) 
		WaitForSingleObject(hThread2, INFINITE); 

	PrintStatistics();

exit:;
	// close socket
	myping.CloseSocket();
	myICMP::TerminateLibrary();
	printf("\n");
	return ProgramExitValue;
}



void PrintStatistics()
{
	// print statistics
	printf(	"\n"
			"Ping statistics for %s\n"
			"  Packets: Sent = %ld, Received = %ld, Lost = %ld (%.0lf%% loss),\n",
			HostIP,
			(long)SendCount, (long)ReceiveCount, (long)(SendCount-ReceiveCount), SendCount>0? (SendCount-ReceiveCount)*100/(double)SendCount : 0);
	if (ElapsedMin >= 0 && ReceiveCount > 0) {
		char jitterText[50]{ 0 };
		if (JitterCount > 1) {
			// added by JJB: Calculate Jitter (aka Variance defined as avarage squared difference from the Mean or Average)
			double TempAverage = 0.0;
			double TempVariance = 0.00 ;
			for (int i=0; i<JitterCount ; i++ ) {
					TempAverage += JitterArray[i];
			}
			TempAverage /= JitterCount;
			for (int i=0; i<JitterCount ; i++ ) {
					TempVariance += (JitterArray[i] - TempAverage) * (JitterArray[i] - TempAverage) ;
			}
			double Jitter = sqrt(TempVariance/(JitterCount-1));

			//TempVariance=0.00;
			//for (i=0; i<JitterCount ; i++ ) {
			//		TempVariance += (JitterArray[i] - ElapsedMin) * (JitterArray[i] - ElapsedMin) ;
			//}
			// double JitterNominal = sqrt(TempVariance/(JitterCount-1));
			// added by JJB: end Jitter calculation
			sprintf(jitterText, ", Jitter Statistical = %.2lfms",Jitter);
		}
		printf("Approximate round trip times in milli-seconds:\n"
			"  Minimum = %.2lfms, Maximum = %.2lfms, Average = %.2lfms%s\n",
			(double)ElapsedMin, (double)ElapsedMax, (double)(ElapsedAverage / ReceiveCount), jitterText);
        if (JitterCount < ReceiveCount) {
            printf( "*** Jitter (Latency Variance) is calculated on the first %ld packets of the %ld packets received ***\n", (long)JitterCount, (long)ReceiveCount);
		}
	}
}


DWORD WINAPI ReceiveFunc(VOID) 
{
	myICMP	myping;
	static int i=0;
	if (!myping.OpenSocket()) 
		goto exit;
	if (!myping.ListenSetup(Timeout))
		goto exit;
	myping.SetPrintTrace(PrintTraceStyle);
	myping.SetPrintTimeStampOption(FlagPrintTimeStamp);

	while (Status != STATUS_EXIT) {
		// Attendi che la palla tocca a noi
		while (Status != STATUS_LISTENER) {
			if (Status == STATUS_EXIT)
				break;
			Sleep(1);
		}
		// Passiamo la palla al sender e quindi stiamo in ascolto
		if (Status == STATUS_EXIT)
			break;
		Status = STATUS_SENDER;
		// Stai in ascolto
		int result =myping.Listen(&GoodTimer,&WaitSequence,NULL,NULL);
		if (result < 0)
			goto exit;
		// Aggiorna statistiche se abbiamo ricevuto un pacchetto
		if (result > 0) {
			if (SkipFirstPackets > 0) {
				--SkipFirstPackets;
				--SendCount;
			} else {
				double t = myping.GetElapsedTime();
				if (ElapsedMin < 0) {
					ElapsedMin = ElapsedMax = ElapsedAverage = JitterArray[JitterCount++] = t;
				} else {
					if (ElapsedMin > t) ElapsedMin = t;
					if (ElapsedMax < t) ElapsedMax = t;
					ElapsedAverage = ElapsedAverage + t;
                    if (JitterCount < JITTER_ARRAY_SIZE) {
                        JitterArray[JitterCount++] = t;
                    }
				}
				++ReceiveCount;
			}
		}
	}
exit:;
	Status = STATUS_EXIT;
	return 0; 
} 



void PrintUsage()
{
	printf(
		"\n"
		"Usage: tping [Options] destination-host\n"
		"\n"
		"Standard Ping Options:\n"
		"    -t             Ping the specified host until stopped.\n"
//		"                   To see statistics and continue - type Control-Break;\n"
		"                   To stop - type Control-C.\n"
		"    -a             Resolve addresses to hostnames.\n"
		"    -n count       Number of echo requests to send.\n"
		"    -l size        Send buffer size.\n"
		"    -f             Set Don't Fragment flag in packet.\n"
		"    -i TTL         Time To Live.\n"
		"    -v TOS         Type of service.\n"            
		//   -r count       Record route for count hops.
		//	 -s count       Timestamp for count hops.
		//   -j host-list   Loose source route along host-list.
		//   -k host-list   Strict source route along host-list.
		"    -w timeout     Timeout in milliseconds to wait for each reply.\n"
		"    -S srcaddr     Source address to use.\n"
		"\n"
		"True Ping Extended Options:\n"
		"    -j             A very simple trace route (Time To Live will \n"
		"                   increment at each ping).\n"
		"    -p             Do not use another thread for receiving data \n"
		"                   (in real applications multiple threads are used).\n" 
		"    -c             Ping compatibility: Use standard data buffer: \n"
		"                   'abcdefghijklmnopqrstuvw' (default is random data buffer).\n"
		"                   Due to compression of ip packets the standard ping buffer \n"
		"                   does not give realistic results.\n"
		"    -d delay       Delay in milliseconds between a receive and the next send \n"
		"                   0 for no delay. Use \"tping host -n 100 -d 0 > out.txt\" \n" 
		"                   for a high data send rate.\n"            
		"    -z count       Number of echo responses to ignore in the statistic \n"
		"                   calculation. Useful if the first packets are much slower \n"
		"                   than the average packet\n"
		"    -h             Print also the Timestamp for every ping\n"
		"\n"
		"True Ping 1.16 - by Andrea Denzler :) - http://www.AndreaPlanet.com/tping/\n"
		"\n"
		);
}

void FillDataBuffer(char *buffer, int size, BOOL FlagPingCompatible)
{
	if (FlagPingCompatible == TRUE) {
		// Use standard ping data buffer: abcdefghijklmnopqrstuvw
		for (int i=0; i<size; i++)
			buffer[i]=(char)('a' +  i%23);
	} else {
		// Completely random data everytime
		for (int i=0; i<size; i++)
			buffer[i]=(char)rand();
	}
}


DWORD WINAPI DoNothing()
{
	while (Status == STATUS_EXIT) {
		Sleep(1);
	}
	return 0;
}